using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CSharpSample
{
    public partial class Form1 : Form
    {
        //create a connection used in through out the form class
        private System.Data.OleDb.OleDbConnection mConn
            = new System.Data.OleDb.OleDbConnection();

        public Form1()
        {
            InitializeComponent();

            try
            {
                mConn.ConnectionString = @"Provider=OleSQLite.SQLiteSource.1;"
                    + @"Data Source=C:\customer.dat";
                mConn.Open();
                if (HasCustTable())
                {
                    this.btnRead.Enabled = true;
                }
            }
            catch (System.Data.OleDb.OleDbException ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            catch (System.Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }


        }

        //Synopsis:
        //1. Create a table named cust if it does not exist. If the table exits,
        //delete all records from the table.
        //2. Insert some records into cust table
        //3. Update the records using parameters which makes it easy for
        //binary data manipulation		
        private void btnCreate_Click(object sender, EventArgs e)
        {
            try
            {
                System.Data.OleDb.OleDbCommand cmd
                    = new System.Data.OleDb.OleDbCommand();
                cmd.Connection = mConn;
                //delete all data from the table if table exists
                if (HasCustTable())
                {
                    cmd.CommandText = "DELETE FROM cust;";
                }
                else   //create cust table if it does not exists
                {
                    cmd.CommandText = "CREATE TABLE cust(ID "
                        + "INTEGER PRIMARY KEY, Name VARCHAR(60), "
                        + "Address VARCHAR(120), Picture BLOB);";
                }
                cmd.ExecuteNonQuery();

                cmd.CommandText = "";

                //insert data
                int I;
                for (I = 100; I <= 110; I++)
                {
                    cmd.CommandText = cmd.CommandText
                    + "INSERT INTO cust (ID, Name) VALUES ("
                    + I.ToString() + ", 'Sean " + I.ToString() + "');";
                }
                cmd.ExecuteNonQuery();
                //enable the read button once the table exists
                this.btnRead.Enabled = true;

                //update records with Parameters
                byte[] BA = new byte[1000];
                for (I = 0; I < 1000; I++)
                {
                    BA[I] = (byte)(I + 1);
                }
                System.Data.OleDb.OleDbParameter[] oParams
                    = {new System.Data.OleDb.OleDbParameter(), 
			   new System.Data.OleDb.OleDbParameter(), 
			   new System.Data.OleDb.OleDbParameter()};
                oParams[0].Value = 100;
                oParams[1].Value = "1010 Main St, Salem, OR";
                oParams[2].Value = BA;
                cmd.CommandText =
                    "UPDATE cust SET Address=?, Picture=? WHERE ID=?;";

                //add parameters according to commend text
                cmd.Parameters.Add(oParams[1]);
                cmd.Parameters.Add(oParams[2]);
                cmd.Parameters.Add(oParams[0]);
                cmd.ExecuteNonQuery();
                cmd = null;
                MessageBox.Show("Done");
            }
            catch (System.Data.OleDb.OleDbException ex)
            {
                MessageBox.Show(ex.Message);
            }
            catch (System.Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        //Synopsis:
        //1. create an OleDbcommand
        //2. assign command text
        //3. execute command
        //4. read data from db, the results are 
        //  written into the debug output window
        private void btnRead_Click(object sender, EventArgs e)
        {
            try
            {
                System.Data.OleDb.OleDbCommand cmd
                    = new System.Data.OleDb.OleDbCommand();
                cmd.Connection = mConn;
                cmd.CommandText = "SELECT ID, Name, Address FROM cust;";
                System.Data.OleDb.OleDbDataAdapter da = new System.Data.OleDb.OleDbDataAdapter(cmd);
                System.Data.DataTable  dt = new DataTable ();
                da.Fill(dt);
                dataGridView1.DataSource = dt;
                
                System.Data.OleDb.OleDbDataReader oReader = cmd.ExecuteReader();
                while (oReader.Read())
                {
                    System.Diagnostics.Debug.WriteLine(oReader.GetInt32(0));
                    System.Diagnostics.Debug.WriteLine("");
                    System.Diagnostics.Debug.WriteLine(oReader.GetString(1));
                    System.Diagnostics.Debug.WriteLine("");
                    if (!oReader.IsDBNull(2))
                    {
                        System.Diagnostics.Debug.WriteLine(
                                oReader.GetString(2));
                        System.Diagnostics.Debug.WriteLine("");
                    }
                }
                oReader.Close();
            }
            catch (System.Data.OleDb.OleDbException ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            catch (System.Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }
        //Synopsis:
        //check if the cust table exists. Return true if it does. 
        //Return false otherwise.
        private bool HasCustTable()
        {
            System.Data.OleDb.OleDbCommand cmd
                = new System.Data.OleDb.OleDbCommand();
            
            using (cmd)
            {
                DataTable dt =  mConn.GetOleDbSchemaTable(System.Data.OleDb.OleDbSchemaGuid.Tables , null);
                foreach (DataRow r in dt.Rows)
                {
                    if (r.ItemArray.GetValue(2).ToString() == "cust" &&
                        r.ItemArray.GetValue(3).ToString() == "table") return true;
                    return false;
                }
            }
            return false;
        }		
    }
}